<!DOCTYPE html>
<html lang="en">
<head>
    <title>{% block "title"%}{% end %}</title>
    <meta name="api-version" value="2" />
    <link rel="stylesheet" href="/static/css/simple.css">
</head>
<body>
{% block "body" %}{% end %}
</body>
</html>